# s.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
# sys.path.insert(0, os.path.abspath(os.path.join(os.path.dirname(__file__), '..')))
# import dpti
# from . import lib
# import .lib
# from . import equi
# from . import hti
# from . import hti_liq
# from . import ti
# from . import gdi
# from dpti import lib
